@@ -9,7 +9,7 @@ from .websockets import BinanceSocketManager
 class DepthCache(object):
 
     def __init__(self, symbol):
-        """Intialise the DepthCache
+        """Initialise the DepthCache
 
         :param symbol: Symbol to create depth cache for
         :type symbol: string
@@ -18,6 +18,7 @@ class DepthCache(object):
         self.symbol = symbol
         self._bids = {}
         self._asks = {}
+        self.update_time = None
 
     def add_bid(self, bid):
         """Add a bid to the cache
@@ -116,11 +117,10 @@ class DepthCache(object):
         return lst
 
 
-class DepthCacheManager(object):
+class BaseDepthCacheManager(object):
+    DEFAULT_REFRESH = 60 * 30  # 30 minutes
 
-    _default_refresh = 60 * 30  # 30 minutes
-
-    def __init__(self, client, symbol, callback=None, refresh_interval=_default_refresh, bm=None):
+    def __init__(self, client, symbol, callback=None, refresh_interval=None, bm=None, limit=10):
         """Initialise the DepthCacheManager
 
         :param client: Binance API client
@@ -131,51 +131,36 @@ class DepthCacheManager(object):
         :type callback: function
         :param refresh_interval: Optional number of seconds between cache refresh, use 0 or None to disable
         :type refresh_interval: int
+        :param limit: Optional number of orders to get from orderbook
+        :type limit: int
 
         """
         self._client = client
         self._symbol = symbol
+        self._limit = limit
         self._callback = callback
         self._last_update_id = None
         self._depth_message_buffer = []
         self._bm = bm
-        self._depth_cache = DepthCache(self._symbol)
-        self._refresh_interval = refresh_interval
+        self._refresh_interval = refresh_interval or self.DEFAULT_REFRESH
         self._conn_key = None
 
         self._start_socket()
         self._init_cache()
 
     def _init_cache(self):
-        """Initialise the depth cache calling REST endpoint
+        """Initialise the depth cache and set a refresh time
 
         :return:
         """
-        self._last_update_id = None
-        self._depth_message_buffer = []
 
-        res = self._client.get_order_book(symbol=self._symbol, limit=500)
-
-        # process bid and asks from the order book
-        for bid in res['bids']:
-            self._depth_cache.add_bid(bid)
-        for ask in res['asks']:
-            self._depth_cache.add_ask(ask)
-
-        # set first update id
-        self._last_update_id = res['lastUpdateId']
+        # initialise or clear depth cache
+        self._depth_cache = DepthCache(self._symbol)
 
         # set a time to refresh the depth cache
         if self._refresh_interval:
             self._refresh_time = int(time.time()) + self._refresh_interval
 
-        # Apply any updates from the websocket
-        for msg in self._depth_message_buffer:
-            self._process_depth_message(msg, buffer=True)
-
-        # clear the depth buffer
-        del self._depth_message_buffer
-
     def _start_socket(self):
         """Start the depth cache socket
 
@@ -184,13 +169,12 @@ class DepthCacheManager(object):
         if self._bm is None:
             self._bm = BinanceSocketManager(self._client)
 
-        self._conn_key = self._bm.start_depth_socket(self._symbol, self._depth_event)
+        self._conn_key = self._get_conn_key()
         if not self._bm.is_alive():
             self._bm.start()
 
-        # wait for some socket responses
-        while not len(self._depth_message_buffer):
-            time.sleep(1)
+    def _get_conn_key(self):
+        raise NotImplementedError
 
     def _depth_event(self, msg):
         """Handle a depth event
@@ -208,11 +192,7 @@ class DepthCacheManager(object):
             if self._callback:
                 self._callback(None)
 
-        if self._last_update_id is None:
-            # Initial depth snapshot fetch not yet performed, buffer messages
-            self._depth_message_buffer.append(msg)
-        else:
-            self._process_depth_message(msg)
+        self._process_depth_message(msg)
 
     def _process_depth_message(self, msg, buffer=False):
         """Process a depth event message.
@@ -222,29 +202,24 @@ class DepthCacheManager(object):
 
         """
 
-        if buffer and msg['u'] <= self._last_update_id:
-            # ignore any updates before the initial update id
-            return
-        elif msg['U'] != self._last_update_id + 1:
-            # if not buffered check we get sequential updates
-            # otherwise init cache again
-            self._init_cache()
-
         # add any bid or ask values
         for bid in msg['b']:
             self._depth_cache.add_bid(bid)
         for ask in msg['a']:
             self._depth_cache.add_ask(ask)
 
+        # keeping update time
+        self._depth_cache.update_time = msg['E']
+
         # call the callback with the updated depth cache
         if self._callback:
             self._callback(self._depth_cache)
 
-        self._last_update_id = msg['u']
-
         # after processing event see if we need to refresh the depth cache
         if self._refresh_interval and int(time.time()) > self._refresh_time:
+            self.close()
             self._init_cache()
+            self._start_socket()
 
     def get_depth_cache(self):
         """Get the current depth cache
@@ -264,3 +239,125 @@ class DepthCacheManager(object):
             self._bm.close()
         time.sleep(1)
         self._depth_cache = None
+
+    def get_symbol(self):
+        """Get the symbol
+
+        :return: symbol
+        """
+        return self._symbol
+
+
+class DepthCacheManager(BaseDepthCacheManager):
+
+    def __init__(self, client, symbol, callback=None, refresh_interval=None, bm=None, limit=500,
+                 ws_interval=None):
+        """Initialise the DepthCacheManager
+
+        :param client: Binance API client
+        :type client: binance.Client
+        :param symbol: Symbol to create depth cache for
+        :type symbol: string
+        :param callback: Optional function to receive depth cache updates
+        :type callback: function
+        :param refresh_interval: Optional number of seconds between cache refresh, use 0 or None to disable
+        :type refresh_interval: int
+        :param limit: Optional number of orders to get from orderbook
+        :type limit: int
+        :param ws_interval: Optional interval for updates on websocket, default None. If not set, updates happen every second. Must be 0, None (1s) or 100 (100ms).
+        :type ws_interval: int
+
+        """
+        self._ws_interval = ws_interval
+        self._last_update_id = None
+        self._depth_message_buffer = []
+        super().__init__(client, symbol, callback, refresh_interval, bm, limit)
+
+    def _init_cache(self):
+        """Initialise the depth cache calling REST endpoint
+
+        :return:
+        """
+        self._last_update_id = None
+        self._depth_message_buffer = []
+
+        res = self._client.get_order_book(symbol=self._symbol, limit=self._limit)
+
+        # initialise or clear depth cache
+        super()._init_cache()
+
+        # process bid and asks from the order book
+        for bid in res['bids']:
+            self._depth_cache.add_bid(bid)
+        for ask in res['asks']:
+            self._depth_cache.add_ask(ask)
+
+        # set first update id
+        self._last_update_id = res['lastUpdateId']
+
+        # Apply any updates from the websocket
+        for msg in self._depth_message_buffer:
+            self._process_depth_message(msg, buffer=True)
+
+        # clear the depth buffer
+        self._depth_message_buffer = []
+
+    def _start_socket(self):
+        """Start the depth cache socket
+
+        :return:
+        """
+        super()._start_socket()
+
+        # wait for some socket responses
+        while not len(self._depth_message_buffer):
+            time.sleep(1)
+
+    def _get_conn_key(self):
+        return self._bm.start_depth_socket(self._symbol, self._depth_event, interval=self._ws_interval)
+
+    def _process_depth_message(self, msg, buffer=False):
+        """Process a depth event message.
+
+        :param msg: Depth event message.
+        :return:
+
+        """
+
+        if self._last_update_id is None:
+            # Initial depth snapshot fetch not yet performed, buffer messages
+            self._depth_message_buffer.append(msg)
+            return
+
+        if buffer and msg['u'] <= self._last_update_id:
+            # ignore any updates before the initial update id
+            return
+        elif msg['U'] != self._last_update_id + 1:
+            # if not buffered check we get sequential updates
+            # otherwise init cache again
+            self._init_cache()
+
+        # add any bid or ask values
+        for bid in msg['b']:
+            self._depth_cache.add_bid(bid)
+        for ask in msg['a']:
+            self._depth_cache.add_ask(ask)
+
+        # keeping update time
+        self._depth_cache.update_time = msg['E']
+
+        # call the callback with the updated depth cache
+        if self._callback:
+            self._callback(self._depth_cache)
+
+        self._last_update_id = msg['u']
+
+        # after processing event see if we need to refresh the depth cache
+        if self._refresh_interval and int(time.time()) > self._refresh_time:
+            self._init_cache()
+
+
+class OptionsDepthCacheManager(BaseDepthCacheManager):
+
+    def _get_conn_key(self):
+        return self._bm.start_options_depth_socket(self._symbol, self._depth_event)
